@@ -81,6 +81,7 @@ $(function() {
 
 	var diffEditor = CodeMirror.fromTextArea(document.getElementById('diff'), {
 		//TODO these should be configurable!
+		
 		tabSize: 4,
 		indentUnit: 4,
 		indentWithTabs: true,
@@ -89,7 +90,16 @@ $(function() {
 
 	// Some friendly advice
 	diffEditor.setValue("This will contain commit changes.\nPlease use Git Diff to show something useful");
-	
+
+	// Create the readonly CodeMirror search editor instance
+	var searchEditor = CodeMirror.fromTextArea(document.getElementById('search'), {
+		//TODO these should be configurable!
+		tabSize: 4,
+		indentUnit: 4,
+		indentWithTabs: true,
+		readOnly: true
+	});
+
 	$('a[href="#input-tab"]').click(function() { 
 		setTimeout(function() {
 			inputEditor.refresh();
@@ -103,109 +113,49 @@ $(function() {
 		}, 0);
 	});
 
+	$('a[href="#search-tab"]').click(function() { 
+		setTimeout(function() {
+			searchEditor.refresh();
+		}, 0);
+	});
+
 	$('a[href="#diff-tab"]').click(function() { 
+		$(document).trigger("action-git-diff");
 		setTimeout(function() {
 			diffEditor.refresh();
 		}, 0);
 	});
 
-	var messages = [];
 
-	window.sendMessage = function(action, params, callback) {
-		var message = {
-			action   : action,
-			params   : params,
-			callback : callback
-		};
-		console.info('sendMessage(' + action + ", ...)");
-		messages.push(message);
-		ws.send(JSON.stringify(message));
-	};
+	var firstTime = true;
 
-	//--- Websockets support
-	if("WebSocket" in window) {
-		// WebSockets is supported
-		console.info("websockets supported");
+	setInterval(function() {
+		$.post("/ping", {}, function() {
+			// Green connection status
+			$("#connection-status")
+			.text("Connected")
+			.addClass("btn-success")
+			.removeClass("btn-danger");
 
+			if(firstTime) {
+				// Add an empty one editor tab on startup
+				AddEditorTab();
 
-		// Bug workaround: Firefox escape key kills the websocket connection...
-		if(navigator.userAgent.indexOf("Firefox") != -1) {
-			$(window).keydown(function(e) {
-				// Check for escape key
-				if (e.which == 27) {
-					// The following seems to fix the symptom but only in case the document has the focus
-					e.preventDefault();
-				}
-			});	
-		}
+				// Set diff editor mode
+				window.ChangeMode(diffEditor, 'diff');
 
-		var firstTime = true;
-		var connectWebSocket = function() {
-			// Connect
-			window.ws = new WebSocket('ws://' + window.location.host + '/websocket');
-
-			// Called once when the websocket connection is opened
-			ws.onopen = function () { 
-				// Change connection status to connected green
-				$("#connection-status")
-					.text("Connected")
-					.addClass("btn-success")
-					.removeClass("btn-danger");
-	
-				if(firstTime) {
-					// Add an empty one editor tab on startup
-					AddEditorTab();
-					firstTime = false;
-					
-					// Set diff editor mode
-					window.ChangeMode(diffEditor, 'diff');
-				}
-			};
-
-			// Called when the websocket message arrives
-			ws.onmessage = function(e) {
-				var message = messages.pop();
-				if(message.callback) {
-					var data = JSON.parse(e.data);
-					message.callback(data);
-				}
-			};
-
-			// Called once when the websocket connection is closed
-			ws.onclose = function(e) {
-
-				// Change connection status to closed red
-				$("#connection-status")
-					.text("Closed")
-					.addClass("btn-danger")
-					.removeClass("btn-success");
-				
-				// Cleanup old websocket connection
-				if(window.ws) {
-					window.ws.close();
-				}
-
-				// Reconnect...
-				setTimeout(connectWebSocket, 500);
-			};
-		};
-
-		// Connect websocket connection
-		connectWebSocket();
-
-	} else {
-		// No WebSockets support
-		alert("WebSockets is not supported here!\nBrowser: " + navigator.userAgent);
-
-		// Change connection status to unsupported red
-		$("#connection-status")
-			.text("Unsupported")
+				firstTime = false;
+			}
+		}).fail(function() {
+			// Red connection status
+			$("#connection-status")
+			.text("Closed")
 			.addClass("btn-danger")
 			.removeClass("btn-success");
-	}
-	//--- Websockets support
 
-	
+		});
+	}, 5000);
+ 
 
 	function displayHelp(cm) {
 		var selection = cm.getSelection();
@@ -290,6 +240,40 @@ $(function() {
 
 		});
 
+		var show_cmd_output = function(data) {
+			var outputEditor = GetOutputEditor();
+			outputEditor.setValue('');
+
+			if(typeof data.stderr == "undefined") {
+				// Websocket messages are not synchronized... workaround for now
+				outputEditor.setValue("Invalid response.. Please run it again. Hopefully will fix it soon :)");
+
+				// Show output tab
+				$('a[href="#output-tab"]').tab('show');
+
+				setTimeout(function() {
+					outputEditor.refresh();
+				}, 0);
+
+				return;
+			}
+
+			if (data.stderr && data.stderr.length) {
+				// Handle STDERR
+				outputEditor.setValue(data.stderr + "\n");
+			}
+
+			// Handle STDOUT
+			outputEditor.setValue(outputEditor.getValue() + data.stdout + "\nExit code: " + data.exit);
+
+			// Show output tab
+			$('a[href="#output-tab"]').tab('show');
+
+			setTimeout(function() {
+				outputEditor.refresh();
+			}, 0);
+		};
+
 		// Create the CodeMirror editor instance
 		var editor = CodeMirror.fromTextArea(document.getElementById(editorId), {
 			//TODO these should be configurable!
@@ -300,15 +284,62 @@ $(function() {
 			indentWithTabs: true,
 			highlightSelectionMatches: true,
 			styleSelectedText: true,
-			autoCloseBrackets: true,
 			styleActiveLine: true,
 			flattenSpans: false,
 			extraKeys: {
-				"F1": function(cm) {
-					displayHelp(cm);
+				//"F1": function(cm) {
+				//	displayHelp(cm);
+				//},
+				'Ctrl-Alt-F': function(cm) {
+					$(document).trigger('action-ack');
 				},
 				'Alt-A': function(cm) {
 					$(".actions-button").click();
+				},
+				'Alt-N': function(cm) {
+					$(document).trigger('action-new-file');
+				},
+				'Alt-O': function(cm) {
+					$(document).trigger('action-open-file');
+				},
+				'Alt-S': function(cm) {
+					$(document).trigger('action-save-file');
+				},
+				'Alt-W': function(cm) {
+					$(document).trigger('action-close-file');
+				},
+				"Alt-L": function(cm) {
+					$(document).trigger('action-goto-line');
+				},
+				'Alt-Enter': function(cm) {
+					// TODO make it more generic and we need runtime profiles
+					var mode = cm.getOption("mode");
+					if(mode == "perl") {
+						$.post(
+							'run_perl',
+							{
+								"source": cm.getValue(),
+								"input": GetInputEditor().getValue()
+							},
+							function(result) {
+								show_cmd_output(result);
+							}
+						);
+					} else {
+						$("#action-run").click();
+					}
+				},
+				"F10": function(cm) {
+					// F11 does not work on linux for some reason!
+					cm.setOption("fullScreen", !cm.getOption("fullScreen"));
+				},
+				"F11": function(cm) {
+					cm.setOption("fullScreen", !cm.getOption("fullScreen"));
+				},
+				"Esc": function(cm) {
+					if (cm.getOption("fullScreen")) {
+						cm.setOption("fullScreen", false);
+					}
 				}
 			}
 		});
@@ -336,12 +367,20 @@ $(function() {
 
 		// Hook up with change event
 		var timeoutId;
-		editor.on("change", function(cm) {
+		var onChange = function(cm) {
 				clearInterval(timeoutId);
 				timeoutId = setTimeout(function() {
 					var mode = cm.getOption("mode");
 					if (mode == "perl") {
 						syntaxCheck(cm);
+
+						if($("#preview-tab").is(":visible")) {
+							$('a[href="#preview-tab"]').click();
+						}
+					} else if(mode == "markdown") {
+						if($("#preview-tab").is(":visible")) {
+							$('a[href="#preview-tab"]').click();
+						}
 					} else if (mode == "javascript") { 
 						$(document).trigger('action-jshint');
 					} else {
@@ -351,7 +390,10 @@ $(function() {
 				}, 250);
 
 				showStarAfterNameIfModified(cm);
-		});
+		};
+
+		editor.on("change", onChange);
+		editor.on("focus", onChange);
 
 		
 		// Store editor reference
@@ -395,12 +437,27 @@ $(function() {
 			if(mode == "perl") {
 				// Run syntax check at startup
 				syntaxCheck(editor);
+
+				// Update preview tab if it is visible
+				if($("#preview-tab").is(":visible")) {
+					$('a[href="#preview-tab"]').click();
+				}
+
+			} else if(mode == "markdown") {
+				if($("#preview-tab").is(":visible")) {
+					$('a[href="#preview-tab"]').click();
+				}
 			} else if (mode == "javascript") { 
 				$(document).trigger('action-jshint');
 			} else {
 				$("#pod").html('Not supported for mode "' + mode + '"');
 			}
-			
+
+			// Update changes tab if it is visible
+			if($("#diff-tab").is(":visible")) {
+				$(document).trigger("action-git-diff");
+			}
+
 			window.showWhitespace(editor);
 
 			editor.refresh();
@@ -417,7 +474,7 @@ $(function() {
 	});
 	
 	function _displayHelp(topic, bShowDialog) {
-		window.sendMessage('help_search', {"topic": topic}, function(results) {
+		$.post('/help_search', {"topic": topic}, function(results) {
 			if(results.length > 0) {
 				$(".topic").val(topic);
 				var html = '';
@@ -473,14 +530,42 @@ $(function() {
 		});
 	};
 
-	var podChanged = function(editor) {
-		window.sendMessage('pod2html', {"source": editor.getValue()}, function(html) {
-			$('#pod').html(html);
-		});
+	var previewTabChanged = function(editor) {
+		var mode = editor.getOption("mode");
+		if(mode == "perl") {
+			$.post("/pod2html", 
+				{
+					"source": editor.getValue(),
+					"style": $("#pod_style_selector", $('#options-dialog')).val()
+				},function(html) {
+					var $iframe = $("#pod");
+					var scrollTop = $iframe.contents().scrollTop();
+					var doc = $iframe[0].contentDocument;
+					doc.open();
+					doc.write(html);
+					doc.close();
+					$iframe.contents().scrollTop(scrollTop);
+				}
+			);
+		} else if(mode == "markdown") {
+			$.post("/md2html", 
+				{
+					"text": editor.getValue()
+				},function(html) {
+					var $iframe = $("#pod");
+					var scrollTop = $iframe.contents().scrollTop();
+					var doc = $iframe[0].contentDocument;
+					doc.open();
+					doc.write(html);
+					doc.close();
+					$iframe.contents().scrollTop(scrollTop);
+				}
+			);
+		}
 	};
 
-	$('a[href="#pod-tab"]').click(function() { 
-		podChanged(GetCurrentEditor());
+	$('a[href="#preview-tab"]').click(function() { 
+		previewTabChanged(GetCurrentEditor());
 	});
 
 	// Get the current editor
@@ -501,6 +586,10 @@ $(function() {
 		return inputEditor;
 	};
 
+	window.GetSearchEditor = function() {
+		return searchEditor;
+	};
+
 	window.GetEditors = function() {
 		var results = [];
 		for(var editorId in editors) {
@@ -519,7 +608,7 @@ $(function() {
 		var editor = GetCurrentEditor();
 		var outputEditor = GetOutputEditor();
 		var source = editor.getValue();
-		window.sendMessage('perl-tidy', {"source": source}, function(data) {
+		$.post('perl_tidy', {"source": source}, function(data) {
 			if(data.error === '') {
 
 				if(source != data.source) {
@@ -542,7 +631,7 @@ $(function() {
 
 	var syntaxCheckerWidgets = [];
 	var syntaxCheck = function(editor) {
-		window.sendMessage('syntax-check', {"source": editor.getValue()}, function(problems) {
+		$.post('/syntax_check', {"source": editor.getValue()}, function(problems) {
 			editor.operation(function(){
 				var i;
 				for (i = 0; i < syntaxCheckerWidgets.length; i++) {
@@ -622,10 +711,6 @@ $(function() {
 		});
 	};
 
-	$(document).on('action-syntax-check', function() {
-		syntaxCheck(GetCurrentEditor());
-	});
-
 	// JSHint: JavaScript static code analysis
 	var jsHintWidgets = [];
 	var jsHint = function(editor) {
@@ -752,7 +837,7 @@ $(function() {
 
 	$(".topic").typeahead({
 		source : function(query, process) {
-			window.sendMessage('/typeahead', {'query': query}, function(matches) {
+			$.post('/typeahead', {'query': query}, function(matches) {
 				process(matches);
 			});
 		}
@@ -811,6 +896,8 @@ $(function() {
 
 			// Update currentEditorId
 			currentEditorId = "editor" + $lastTab.attr("href").substring(1);
+
+			GetCurrentEditor().focus();
 		}
 	});
 
@@ -830,10 +917,16 @@ $(function() {
 	$(document).on('action-save-file', function() {
 		var editor = GetCurrentEditor();
 		var filename = GetCurrentFilename();
-		window.sendMessage('save-file', {"filename": filename, "source": editor.getValue()}, function(data) {
+		$.post('/save_file', {"filename": filename, "source": editor.getValue()}, function(data) {
 			if(data.err === '') {
 				editor.markClean();
+
 				showStarAfterNameIfModified(editor);
+
+				// Update changes tab if it is visible
+				if($("#diff-tab").is(":visible")) {
+					$(document).trigger("action-git-diff");
+				}
 			} else {
 				alert('Error:\n' + data.err);
 			}
@@ -854,7 +947,7 @@ $(function() {
 			source = editor.getValue();
 		}
 
-		sendMessage('dump-ppi-tree', {"source": source}, function(data) {
+		$.post('/dump_ppi_tree', {"source": source}, function(data) {
 
 			if(data.error === '') {
 				outputEditor.setValue(data.output);
@@ -873,24 +966,9 @@ $(function() {
 		$(document).trigger("action-about");
 	});
 	
-	$(document).on('action-debug-step-in', function() {
-		alert("Not implemented yet");
-	});
-
-	$(document).on('action-debug-step-over', function() {
-		alert("Not implemented yet");
-	});
-	
-	$(document).on('action-debug-step-out', function() {
-		alert("Not implemented yet");
-	});
-
-	$(document).on('action-debug-stop', function() {
-		alert("Not implemented yet");
-	});
 
 	$(document).on('action-git-diff', function() {
-		window.sendMessage('git-diff', {}, function(result) {
+		$.post('/git_diff', {}, function(result) {
 
 			diffEditor.setValue('');
 
@@ -911,7 +989,7 @@ $(function() {
 
 	$(document).on('action-perl-strip', function() {
 		var editor = GetCurrentEditor();
-		window.sendMessage('perl-strip', {source: editor.getValue()}, function(result) {
+		$.post('/perl_strip', {source: editor.getValue()}, function(result) {
 			if(result.error) {
 				editor.setValue(result.source);
 			} else {
@@ -920,14 +998,192 @@ $(function() {
 		});
 	});
 
-	$(document).on('action-spell-check', function() {
+	$(document).on('action-spellunker', function() {
 		var editor = GetCurrentEditor();
-		window.sendMessage('spell-check', {source: editor.getValue()}, function(result) {
-			if(result.error) {
-				editor.setValue(result.source);
-			} else {
-				alert("No result");
+		$.post('/spellunker', {text: editor.getValue()}, function(result) {
+			spellunker(editor);
+		});
+	});
+
+	// Spellunker: Awesome Spell checking in Pure Perl
+	var spellunkerWidgets = [];
+	var spellunker = function(editor) {
+		$.post('/spellunker', {"text": editor.getValue()}, function(problems) {
+			editor.operation(function(){
+				var i;
+				for (i = 0; i < spellunkerWidgets.length; i++) {
+					editor.removeLineWidget(spellunkerWidgets[i]);
+				}
+				spellunkerWidgets.length = 0;
+
+				//TODO temporary solution until we can figure out how to cooperate
+				$(".farabi-error-icon").remove();
+
+				problemCount = problems.length;
+
+				var html = '';
+
+				if(problems.length > 0) {
+
+					html += "<thead>" +
+						"<th>Line</th>" +
+						"<th>Message</th>" +
+						"<th>File</th>" +
+						"<th>Source</th>" +
+						"</thead>";
+					html += "<tbody>";
+
+					var showProblemsTab = function() {
+						$('a[href="#problems-tab"]').tab('show');
+					};
+
+					for(i = 0; i < problems.length; i++) {
+						var problem = problems[i];
+
+						// Add warning or error under the editor line
+						var msg = document.createElement("div");
+						var icon = msg.appendChild(document.createElement("span"));
+						icon.innerHTML = "!!";
+						icon.className = "farabi-error-icon";
+						msg.appendChild(document.createTextNode(problem.message));
+						msg.className = "farabi-error";
+
+						spellunkerWidgets.push(
+							editor.addLineWidget(
+								problem.line - 1,
+								msg, 
+								{coverGutter: true, noHScroll: true}
+							)
+						);
+
+						$(msg).click(showProblemsTab);
+
+						html += 
+							"<tr>" +
+								'<td class="problem-line">' + problem.line + "</td>" +
+								"<td>" + problem.message + "</td>" +
+								"<td>" + problem.file + "</td>" +
+								"<td>Spellunker</td>" +
+							"</tr>";
+					}
+
+					html += "</tbody>";
+				} else {
+					html = '<tr>' +
+						'<td><span class="badge badge-success">No problems found</span></td>' +
+						'</tr>';
+				}
+
+				$("#problems > table").html(html);
+				$("#problems>  table > tbody > tr").click(function() { 
+					var line = parseInt($(".problem-line", this).text(), 10);
+					var editor = GetCurrentEditor();
+					editor.setCursor({line: line - 1, ch: 0});
+				});
+
+				// Update editor statistics
+				window.showEditorStats(editor);
+			});
+
+		});
+	};
+
+
+	$(document).on('action-ack', function() {
+		var editor = GetCurrentEditor();
+		var selection = editor.getSelection();
+		if(!selection) {
+			setTimeout(function() {
+				alert("Please select some text to search for it in your current project folder");
+			}, 0);
+			return;
+		}
+
+		$.post('/ack', {text: selection}, function(result) {
+
+			var searchEditor = GetSearchEditor();
+	
+			searchEditor.setValue('');
+
+			// Handle STDERR
+			if (result.stderr.length) {
+				searchEditor.setValue(result.stderr + "\n");
 			}
+
+			// Handle STDOUT
+			searchEditor.setValue(searchEditor.getValue() + result.stdout);
+
+			// Show search tab
+			$('a[href="#search-tab"]').tab('show');
+
+			searchEditor.refresh();
+
+		});
+	});
+
+	$(document).on('action-midgen', function() {
+		$.post('/midgen', {}, function(result) {
+
+			var outputEditor = GetOutputEditor();
+
+			outputEditor.setValue('');
+
+			// Handle STDERR
+			if (result.stderr.length) {
+				outputEditor.setValue(result.stderr + "\n");
+			}
+
+			// Handle STDOUT
+			outputEditor.setValue(outputEditor.getValue() + result.stdout);
+
+			// Show output tab
+			$('a[href="#output-tab"]').tab('show');
+
+			outputEditor.refresh();
+		});
+	});
+
+	$(document).on('action-minil-test', function() {
+		$.post('/minil_test', {}, function(result) {
+
+			var outputEditor = GetOutputEditor();
+
+			outputEditor.setValue('');
+
+			// Handle STDERR
+			if (result.stderr.length) {
+				outputEditor.setValue(result.stderr + "\n");
+			}
+
+			// Handle STDOUT
+			outputEditor.setValue(outputEditor.getValue() + result.stdout);
+
+			// Show output tab
+			$('a[href="#output-tab"]').tab('show');
+
+			outputEditor.refresh();
+		});
+	});
+
+	$(document).on('action-dzil-test', function() {
+		$.post('/dzil_test', {}, function(result) {
+
+			var outputEditor = GetOutputEditor();
+
+			outputEditor.setValue('');
+
+			// Handle STDERR
+			if (result.stderr.length) {
+				outputEditor.setValue(result.stderr + "\n");
+			}
+
+			// Handle STDOUT
+			outputEditor.setValue(outputEditor.getValue() + result.stdout);
+
+			// Show output tab
+			$('a[href="#output-tab"]').tab('show');
+
+			outputEditor.refresh();
 		});
 	});
 
